@@ -23,6 +23,7 @@
 
 #include <thunar/thunar-clipboard-manager.h>
 #include <thunar/thunar-gobject-extensions.h>
+#include <thunar/thunar-gdk-extensions.h>
 #include <thunar/thunar-icon-factory.h>
 #include <thunar/thunar-icon-renderer.h>
 #include <thunar/thunar-private.h>
@@ -307,6 +308,89 @@ thunar_icon_renderer_get_size (GtkCellRenderer *renderer,
 
 
 
+static void
+thunar_icon_renderer_color_selected (cairo_t   *cr,
+                                     GtkWidget *widget)
+{
+  cairo_pattern_t *source;
+  GtkStateType     state;
+
+#ifdef CAIRO_BUG_72551_FIXED
+  cairo_save (cr);
+
+  source = cairo_pattern_reference (cairo_get_source (cr));
+  state = gtk_widget_has_focus (widget) ? GTK_STATE_SELECTED : GTK_STATE_ACTIVE;
+  gdk_cairo_set_source_color (cr, &widget->style->base[state]);
+  cairo_set_operator (cr, CAIRO_OPERATOR_MULTIPLY);
+  cairo_mask (cr, source);
+#else /* fallback for RENDER error */
+
+  /* CAIRO_OPERATOR_MULTIPLY */
+  /* causes libx11 error: error_code 2 request_code 155 minor_code 8 */
+  /* with x11 1.0.3, xrender 0.9.1, cairo 1.12.2 */
+  /* cairo_mask (cr, source); */
+
+  cairo_surface_t *target, *image_surface;
+  cairo_t         *image_cr;
+  double           x1, y1, x2, y2;
+
+  cairo_save (cr);
+
+  /* copy xlib surface to an image surface */
+  cairo_clip_extents(cr, &x1, &y1, &x2, &y2);
+  target = cairo_get_target (cr);
+  image_surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, x2, y2);
+  image_cr = cairo_create (image_surface);
+  cairo_set_source_surface(image_cr, target, 0, 0);
+  cairo_set_operator(image_cr, CAIRO_OPERATOR_SOURCE);
+  cairo_paint(image_cr);
+
+  /* render on the image surface */
+  source = cairo_pattern_reference (cairo_get_source (cr));
+  state = gtk_widget_has_focus (widget) ? GTK_STATE_SELECTED : GTK_STATE_ACTIVE;
+  gdk_cairo_set_source_color (image_cr, &widget->style->base[state]);
+  cairo_set_operator (image_cr, CAIRO_OPERATOR_MULTIPLY);
+  cairo_mask (image_cr, source);
+
+  /* copy the image surface back to xlib one and clean up */
+  cairo_set_source_surface (cr, image_surface, 0, 0);
+  cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+  cairo_paint(cr);
+
+  cairo_destroy (image_cr);
+  cairo_surface_destroy (image_surface);
+#endif
+
+  cairo_pattern_destroy (source);
+  cairo_restore (cr);
+}
+
+
+
+static void
+thunar_icon_renderer_color_lighten (cairo_t   *cr,
+                                    GtkWidget *widget)
+{
+  cairo_pattern_t *source;
+
+  cairo_save (cr);
+
+  source = cairo_pattern_reference (cairo_get_source (cr));
+  cairo_set_source_rgb (cr, .15, .15, .15);
+  /* CAIRO_OPERATOR_COLOR_DODGE */
+  /* causes libx11 error: error_code 2 request_code 155 minor_code 8 */
+  /* with x11 1.0.3, xrender 0.9.1, cairo 1.12.2 */
+  /* cairo_set_operator (cr, CAIRO_OPERATOR_COLOR_DODGE); */
+  cairo_set_operator (cr, CAIRO_OPERATOR_ADD);
+
+  cairo_mask (cr, source);
+
+  cairo_pattern_destroy (source);
+  cairo_restore (cr);
+}
+
+
+
 static void
 thunar_icon_renderer_render (GtkCellRenderer     *renderer,
                              GdkWindow           *window,
@@ -325,7 +409,6 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
   GdkRectangle            emblem_area;
   GdkRectangle            icon_area;
   GdkRectangle            draw_area;
-  GtkStateType            state;
   GdkPixbuf              *emblem;
   GdkPixbuf              *icon;
   GdkPixbuf              *temp;
@@ -335,6 +418,9 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
   gint                    position;
   cairo_t                *cr;
   gdouble                 alpha;
+  gint                    emblem_size;
+  gboolean                color_selected;
+  gboolean                color_lighten;
 
   if (G_UNLIKELY (icon_renderer->file == NULL))
     return;
@@ -380,6 +466,10 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
   icon_area.x = cell_area->x + (cell_area->width - icon_area.width) / 2;
   icon_area.y = cell_area->y + (cell_area->height - icon_area.height) / 2;
 
+  /* bools for cairo transformations */
+  color_selected = (flags & GTK_CELL_RENDERER_SELECTED) != 0 && icon_renderer->follow_state;
+  color_lighten = (flags & GTK_CELL_RENDERER_PRELIT) != 0 && icon_renderer->follow_state;
+
   /* create the context */
   cr = gdk_cairo_create (window);
 
@@ -404,25 +494,6 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
         }
       g_object_unref (G_OBJECT (clipboard));
 
-      /* colorize the icon if we should follow the selection state */
-      if ((flags & (GTK_CELL_RENDERER_SELECTED | GTK_CELL_RENDERER_PRELIT)) != 0 && icon_renderer->follow_state)
-        {
-          if ((flags & GTK_CELL_RENDERER_SELECTED) != 0)
-            {
-              state = gtk_widget_has_focus (widget) ? GTK_STATE_SELECTED : GTK_STATE_ACTIVE;
-              temp = exo_gdk_pixbuf_colorize (icon, &widget->style->base[state]);
-              g_object_unref (G_OBJECT (icon));
-              icon = temp;
-            }
-
-          if ((flags & GTK_CELL_RENDERER_PRELIT) != 0)
-            {
-              temp = exo_gdk_pixbuf_spotlight (icon);
-              g_object_unref (G_OBJECT (icon));
-              icon = temp;
-            }
-        }
-
       /* check if we should render an insensitive icon */
       if (G_UNLIKELY (gtk_widget_get_state (widget) == GTK_STATE_INSENSITIVE || !renderer->sensitive))
         {
@@ -443,9 +514,17 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
         }
 
       /* render the invalid parts of the icon */
-      gdk_cairo_set_source_pixbuf (cr, icon, icon_area.x, icon_area.y);
+      thunar_gdk_cairo_set_source_pixbuf (cr, icon, icon_area.x, icon_area.y);
       gdk_cairo_rectangle (cr, &draw_area);
       cairo_paint_with_alpha (cr, alpha);
+
+      /* paint the lighten mask */
+      if (color_lighten)
+        thunar_icon_renderer_color_lighten (cr, widget);
+
+      /* paint the selected mask */
+      if (color_selected)
+        thunar_icon_renderer_color_selected (cr, widget);
     }
 
   /* release the file's icon */
@@ -464,8 +543,11 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
           /* render the emblems */
           for (lp = emblems, position = 0; lp != NULL && position < max_emblems; lp = lp->next)
             {
+              /* calculate the emblem size */
+              emblem_size = MIN ((2 * icon_renderer->size) / 3, 32);
+
               /* check if we have the emblem in the icon theme */
-              emblem = thunar_icon_factory_load_icon (icon_factory, lp->data, icon_renderer->size, FALSE);
+              emblem = thunar_icon_factory_load_icon (icon_factory, lp->data, emblem_size, FALSE);
               if (G_UNLIKELY (emblem == NULL))
                 continue;
 
@@ -474,10 +556,10 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
               emblem_area.height = gdk_pixbuf_get_height (emblem);
 
               /* shrink insane emblems */
-              if (G_UNLIKELY (MAX (emblem_area.width, emblem_area.height) > (gint) MIN ((2 * icon_renderer->size) / 3, 32)))
+              if (G_UNLIKELY (MAX (emblem_area.width, emblem_area.height) > emblem_size))
                 {
                   /* scale down the emblem */
-                  temp = exo_gdk_pixbuf_scale_ratio (emblem, MIN ((2 * icon_renderer->size) / 3, 32));
+                  temp = exo_gdk_pixbuf_scale_ratio (emblem, emblem_size);
                   g_object_unref (G_OBJECT (emblem));
                   emblem = temp;
 
@@ -525,9 +607,17 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
               if (gdk_rectangle_intersect (expose_area, &emblem_area, &draw_area))
                 {
                   /* render the invalid parts of the icon */
-                  gdk_cairo_set_source_pixbuf (cr, emblem, emblem_area.x, emblem_area.y);
+                  thunar_gdk_cairo_set_source_pixbuf (cr, emblem, emblem_area.x, emblem_area.y);
                   gdk_cairo_rectangle (cr, &draw_area);
                   cairo_paint (cr);
+
+                  /* paint the lighten mask */
+                  if (color_lighten)
+                    thunar_icon_renderer_color_lighten (cr, widget);
+
+                  /* paint the selected mask */
+                  if (color_selected)
+                    thunar_icon_renderer_color_selected (cr, widget);
                 }
 
               /* release the emblem */
