@@ -78,6 +78,7 @@ enum
   PROP_MISC_FOLDERS_FIRST,
   PROP_MISC_FULL_PATH_IN_TITLE,
   PROP_MISC_HORIZONTAL_WHEEL_NAVIGATES,
+  PROP_MISC_IMAGE_SIZE_IN_STATUSBAR,
   PROP_MISC_MIDDLE_CLICK_IN_TAB,
   PROP_MISC_RECURSIVE_PERMISSIONS,
   PROP_MISC_REMEMBER_GEOMETRY,
@@ -88,6 +89,7 @@ enum
   PROP_MISC_TAB_CLOSE_MIDDLE_CLICK,
   PROP_MISC_TEXT_BESIDE_ICONS,
   PROP_MISC_THUMBNAIL_MODE,
+  PROP_MISC_FILE_SIZE_BINARY,
   PROP_SHORTCUTS_ICON_EMBLEMS,
   PROP_SHORTCUTS_ICON_SIZE,
   PROP_TREE_ICON_EMBLEMS,
@@ -527,6 +529,20 @@ thunar_preferences_class_init (ThunarPreferencesClass *klass)
                             FALSE,
                             EXO_PARAM_READWRITE);
 
+  /**
+   * ThunarPreferences:misc-image-size-in-statusbar:
+   *
+   * When a single image file is selected, show its size
+   * in the statusbar. This heavily increases I/O in image
+   * folders when moving the selection across files.
+   **/
+  preferences_props[PROP_MISC_IMAGE_SIZE_IN_STATUSBAR] =
+      g_param_spec_boolean ("misc-image-size-in-statusbar",
+                            "MiscImageSizeInStatusbar",
+                            NULL,
+                            FALSE,
+                            EXO_PARAM_READWRITE);
+
   /**
    * ThunarPreferences:misc-middle-click-in-tab:
    *
@@ -660,6 +676,18 @@ thunar_preferences_class_init (ThunarPreferencesClass *klass)
                          THUNAR_THUMBNAIL_MODE_ONLY_LOCAL,
                          EXO_PARAM_READWRITE);
 
+  /**
+   * ThunarPreferences:misc-file-size-binary:
+   *
+   * Show file size in binary format instead of decimal.
+   **/
+  preferences_props[PROP_MISC_FILE_SIZE_BINARY] =
+      g_param_spec_boolean ("misc-file-size-binary",
+                            "MiscFileSizeBinary",
+                            NULL,
+                            FALSE,
+                            EXO_PARAM_READWRITE);
+
   /**
    * ThunarPreferences:shortcuts-icon-emblems:
    *
